
#property description "Daily Range Expert Advisor"
#property version     "3.1"
#property strict

enum DrEaModeEnum
{
   DR_EA_EXPERT = 1,          // EA - Draw           (Slower)
   DR_EA_EXPERT_NO_DRAW = 2,  // EA - No Drawing (Faster)
};

enum LogLevelEnum
{
   LogNone = 0,      // None
   LogError = 1,     // Error
   LogInfo = 2,      // Info
   LogDebug = 3,     // Debug
   LogAll = 4,       // All
};

enum DrBufferIndexEnum
{
   DR_CHANGED = 0,
   DR_ADR = 1,
   DR_CURR = 2,
   DR_OPEN = 3,
   DR_HIGH = 4,
   DR_LOW = 5,
   DR_ROOM_UP = 6,
   DR_ROOM_DN = 7,
};

enum DrEaUpdateEnum
{
   DR_EA_UPDATE_MINIMAL = 0,
   DR_EA_UPDATE_PARTIAL = 1,
   DR_EA_UPDATE_FULL = 2,
};


#ifdef __MQL4__
input string         IndicatorPath = "Market\\Blahtech Daily Range";             // Indicator Path
#else
input string         IndicatorPath = "Market\\Blahtech Daily Range MT5.ex5";     // Indicator Path
#endif
input DrEaModeEnum   ExpertMode = DR_EA_EXPERT_NO_DRAW;                          // Expert Mode
input string         ParameterValues = "";                                       // Parameter Key Value Pairs
input LogLevelEnum   LogLevel = LogError;                                        // Log Level

int    ExtIndicatorHandle;
string ExtParameterValues;


double iCustomDr(DrBufferIndexEnum index, int shift = 0)
{
#ifdef __MQL4__
   return iCustom(NULL, PERIOD_CURRENT, IndicatorPath, ExpertMode, ExtParameterValues, index, shift);
#else
   double arr[1];
   return CopyBuffer(ExtIndicatorHandle, index, shift, 1, arr) > 0 ? arr[0] : 0.0;
#endif
}


int OnInit()
{
   // Example code for passing several parameters
   ExtParameterValues = "values:"
   //"SessionNames=HK,LN,NY;"
   //"SessionTimes=00:00~07:00,  08:00~14:30,  14:30~22:00;"
   //"Range2=6;"
   //"Range3=11;"
   //"Range4=21;"
   //"GMTOffset=0;"
   + ParameterValues;

   Print("DR_EA::OnInit - ExtParameterValues=", ExtParameterValues);
   
#ifdef __MQL4__
   int  ret = INIT_SUCCEEDED;
#else
   ExtIndicatorHandle = iCustom(NULL, PERIOD_CURRENT, IndicatorPath, ExpertMode, ExtParameterValues);
   int  ret = ExtIndicatorHandle > 0 ? INIT_SUCCEEDED : INIT_FAILED;
#endif
   if (LogLevel >= LogInfo) { Print("OnInit - IndicatorPath=", IndicatorPath); }
   return ret;
}


void OnDeinit(const int reason)
{
   #ifndef __MQL4__
   if (ExtIndicatorHandle > 0)
   {
      IndicatorRelease(ExtIndicatorHandle);
   }
   #endif
}


void OnTick()
{
   static int      stTickCount;
   static datetime stPrevTime0;
   datetime        time0 = iTime(NULL, 0, 0);
   int             drChanged = (DrEaUpdateEnum)iCustomDr(DR_CHANGED);

   stTickCount++;
   
   if (drChanged > DR_EA_UPDATE_MINIMAL || time0 != stPrevTime0)
   {
      bool  newBar = time0 != stPrevTime0;
      
      if (LogLevel >= LogDebug) { Print("OnTick TickCount=", stTickCount); }
      
      // For optimal performance - Comment out variables that are not required
      //
      double adr  = iCustomDr(DR_ADR);
      double curr = iCustomDr(DR_CURR);
      double open = iCustomDr(DR_OPEN);
      double high = iCustomDr(DR_HIGH);
      double low  = iCustomDr(DR_LOW);
      double rmUp = iCustomDr(DR_ROOM_UP);
      double rmDn = iCustomDr(DR_ROOM_DN);
      
      if (LogLevel >= LogAll) { Print("OnTick adr=", DoubleToString(adr, _Digits), " curr=", DoubleToString(curr, _Digits)); }
      
      if (newBar)
      {
         // Add user code for new bars
         stPrevTime0 = time0;  // Update prevTime0
      }
      // Add user code for each tick when drChanged
   }
   else if (drChanged == DR_EA_UPDATE_MINIMAL)
   {
      //double rmUp = iCustomDr(DR_ROOM_UP);
      //double rmDn = iCustomDr(DR_ROOM_DN);
      
      // Add user code for minimal changes - RM UP and RM DN will change due to bid price movement
   }
   // Add user code for each tick
}
