
#property description "Market Profile Indi"
#property description "Calling iCustom from another indicator example"
#property version     "2.7"
#property strict

#property indicator_chart_window

// #########################################################################################################
// ## NOTE: This is an indciator and once compiled must be placed under the MT4/Indciators directory to run
// #########################################################################################################

enum MpEaModeEnum
{
   MP_EA_EXPERT = 1,          // EA - Draw           (Slower)
   MP_EA_EXPERT_NO_DRAW = 2,  // EA - No Drawing (Faster)
};

enum LogLevelEnum
{
   LogNone = 0,      // None
   LogError = 1,     // Error
   LogInfo = 2,      // Info
   LogDebug = 3,     // Debug
   LogAll = 4,       // All
};

enum MpBufferIndexEnum
{
   MP_EMPTY = 0,
   MP_CHANGED = 1,
   MP_VAH = 2,
   MP_POC = 3,
   MP_VAL = 4,
   MP_D_VAH = 5,
   MP_D_POC = 6,
   MP_D_VAL = 7,
};

#ifdef __MQL4__
input string         IndicatorPath = "Market\\Blahtech Market Profile";          // Indicator Path
#else
input string         IndicatorPath = "Market\\Blahtech Market Profile MT5.ex5";  // Indicator Path
#endif
input MpEaModeEnum   ExpertMode = MP_EA_EXPERT_NO_DRAW;                          // Expert Mode
input string         LoadParameterFile = "MP_EA_Params.set";                     // Parameter File/Values
//input string         LoadParameterFile = "values:Intervals=3;ActiveSchedules1=1";   // Parameter File/Values
//input string         LoadParameterFile = "values:IntervalPeriod=10080;TpoPeriod=240;Schedule1=00:00 ~ +7 00:00;ScheduleNames1=MP;ActiveSchedules1=1;ProfileLinesTitles=2";                 // Preset 3  - Weeks
//input string         LoadParameterFile = "values:TpoPeriod=60;Schedule1=00:00 ~ +1 00:00;ScheduleNames1=MP;ActiveSchedules1=1;ProfileLinesTitles=2";                                       // Preset 10 - Daily
//input string         LoadParameterFile = "values:IntervalPeriod=43200;TpoPeriod=1440;Schedule1=00:00 ~ +1.0 00:00;ScheduleNames1=MP;ActiveSchedules1=1;ProfileLinesTitles=2";              // Preset 11 - Monthly
//input string         LoadParameterFile = "values:TpoPeriod=240;Schedule1=00:00 ~ +7 00:00;ScheduleNames1=MP;ActiveSchedules1=1;ProfileLinesTitles=2;IntervalMultiplier=7";  // Preset 14 - Rolling 7 Days
input LogLevelEnum   LogLevel = LogError;                                        // Log Level

int    ExtIndicatorHandle;
string ExtLoadParameterFile;


double iCustomMp(MpBufferIndexEnum index, int shift = 0)
{
#ifdef __MQL4__
   return iCustom(NULL, PERIOD_CURRENT, IndicatorPath, ExpertMode, ExtLoadParameterFile, index, shift);
#else
   double arr[1];
   return CopyBuffer(ExtIndicatorHandle, index, shift, 1, arr) > 0 ? arr[0] : 0.0;
#endif
}


int OnInit()
{
   ExtLoadParameterFile = LoadParameterFile;
   
   /*
   Procedure for setting EA Parameters from indicator changes (including preset files)

   1) Drag MP indicator onto "GE 30" Weekly chart
   2) Load parameter set file "MP v2 - Preset 6 - Quarters Of Years - W1 tpo.set"
   3) Set parameter "Profile text / Profile text" => Hidden
   4) Set parameter "Other Parameters / GMT Offset" => 7200
   5) Hit the reset 'r' hotkey
   6) File / Open Data Folder (will open explorer window)
   7) Navigate to MQL4/Files/Blahtech deirectory
   8) Sort files by last modified date
   9) Open most recent MP1_XXXXXXXXXXXXXXX_diff.txt

   The file will contain a list of quoted parameters that have changed from the defaults

   These can be cut/paste directly into your OnInit code following the line...
     ExtLoadParameterFile = "values:"
   */

   // Example code for passing large numbers of parameters
   /*
   ExtLoadParameterFile = "values:"
   "TpoPeriod=240;"
   "Schedule1=00:00 ~ +7 00:00;"
   "ScheduleNames1=MP;"
   "ActiveSchedules1=1;"
   "ProfileLinesTitles=2;"
   "IntervalMultiplier=7;"
   "GMTOffset=7200;";
   /**/
   
  
   
   Print("MP_Indi::OnInit - ExtLoadParameterFile=", ExtLoadParameterFile);
   
#ifdef __MQL4__
   int  ret = INIT_SUCCEEDED;
#else
   ExtIndicatorHandle = iCustom(NULL, PERIOD_CURRENT, IndicatorPath, ExpertMode, ExtLoadParameterFile);
   int  ret = ExtIndicatorHandle > 0 ? INIT_SUCCEEDED : INIT_FAILED;
#endif
   if (LogLevel >= LogInfo) { Print("OnInit - IndicatorPath=", IndicatorPath); }
   return ret;
}


void OnDeinit(const int reason)
{
#ifndef __MQL4__
   if (ExtIndicatorHandle > 0)
   {
      IndicatorRelease(ExtIndicatorHandle);
   }
#endif
}


int OnCalculate(
   const int rates_total,
   const int prev_calculated,
   const datetime &time[],
   const double &open[],
   const double &high[],
   const double &low[],
   const double &close[],
   const long &tick_volume[],
   const long &volume[],
   const int &spread[])
{
   static int      stTickCount;
   static datetime stPrevTime0;
   #ifdef __MQL4__
   datetime        time0 = time[0];
   #else
   datetime        time0 = time[rates_total-1];
   #endif
   int             mpChanged = (int)iCustomMp(MP_CHANGED);  // Only do work when values have changed

   stTickCount++;
#ifndef __MQL4__
   if (stTickCount == 1)
   {
      mpChanged = 2; // Compensate for the extra OnCalculate call during OnInit
   }
#endif   
   if (mpChanged > 0 || time0 != stPrevTime0)
   {
      if (mpChanged > 0 && LogLevel >= LogDebug) { Print("OnCalculate TickCount=", stTickCount, " mpChanged=", mpChanged); }
      
      // For optimal performance - Comment out variables that are not required
      //
      bool     newBar = time0 != stPrevTime0;
      double   vah  = iCustomMp(MP_VAH);
      double   poc  = iCustomMp(MP_POC);
      double   val  = iCustomMp(MP_VAL);
      double   dVah = iCustomMp(MP_D_VAH);
      double   dPoc = iCustomMp(MP_D_POC);
      double   dVal = iCustomMp(MP_D_VAL);
      
      if (LogLevel >= LogAll) { Print("OnCalculate mpChanged=", mpChanged, " vah=", DoubleToString(vah, _Digits), " dVah=", DoubleToString(dVah)); }
      
      if (newBar)
      {
         // Add user code for new bars
         stPrevTime0 = time0;  // Update prevTime0
      }
      // Add user code for each tick when MpChanged
   }
   // Add user code for each tick
   
   return rates_total;
}
