
#property description "Blahtech Supply Demand Expert Advisor - Advanced"
#property version     "3.2"
#property strict

#include "BlahtechSupDemControlAdvanced.mqh"

#ifdef __MQL4__
input string             IndicatorPath = "Market\\Blahtech Supply Demand";          // Indicator Path
#else
input string             IndicatorPath = "Market\\Blahtech Supply Demand MT5.ex5";  // Indicator Path
#endif
input SdEaModeEnum       ExpertMode = SD_EA_EXPERT_NO_DRAW;                         // Expert Mode
input int                MinBarsLevel = 3;                                          // Min Bars For Level
input int                MaxBars = 20000;                                           // Max Bars
input int                ForceRecalcBars = 10000;                                   // Force Recalc Bars
input string             ParameterValues = "";                                      // Parameter Key Value Pairs
//input string             ParameterValues = "ZoneStyle1=2;InfoLevel=1 S;";         // Parameter Key Value Pairs
input MAPPED_TIMEFRAMES  Timeframe1 = MAP_PERIOD_CURRENT;                           // Timeframe 1
input MAPPED_TIMEFRAMES  Timeframe2 = MAP_PERIOD_MAPPED;                            // Timeframe 2
input MAPPED_TIMEFRAMES  Timeframe3 = MAP_PERIOD_MAPPED;                            // Timeframe 3
input SdEaRetouchMode    UpdateRetouches = SD_EA_RETOUCH_FIRST;                     // Update Retouches Mode
input SdEaValidateEnum   ValidateIncremental = SD_EA_VAL_NONE;                      // Validate Incremental
input bool               UseExampleRSI = false;                                     // Use Example RSI Integration Code
input bool               ForceFullUpdate = false;                                   // Force Full Update Every Change
input LogLevelEnum       LogLevel = LogError;                                       // Log Level


SupDemCustomArgs*       ExtCustomArgs1;
SupDemControlAdvanced*  ExtControlAdvanced1;
string                  ExtRsiPath;
int                     ExtRsiHandle;
int                     ExtRsiDirection;


int OnInit()
{
   if (LogLevel >= LogInfo) { Print("OnInit - Start"); }
   
   ExtCustomArgs1 = new SupDemCustomArgs();
   ExtControlAdvanced1 = new SupDemControlAdvanced();
   
   ExtCustomArgs1.IndicatorPath = IndicatorPath;
   ExtCustomArgs1.ExpertMode = ExpertMode;
   ExtCustomArgs1.MinBarsLevel = MinBarsLevel;
   ExtCustomArgs1.MaxBars = MaxBars;
   ExtCustomArgs1.ForceRecalcBars = ForceRecalcBars;
   ExtCustomArgs1.Timeframe1 = Timeframe1;
   ExtCustomArgs1.Timeframe2 = Timeframe2;
   ExtCustomArgs1.Timeframe3 = Timeframe3;
   
   // ###################################################
   // ## Uncomment and/or modify to add parameter values
   // ###################################################
   ExtCustomArgs1.ParameterValues = "values:"
   "Hud=0;"
   "FiltersEnable=1;"
   "FilterMaxLevels=3;"
   "FilterOveralMove1=3.0;"
   //"SummaryMode=0;"
   //"TimeframeVisible2=1;"
   //"ZoneStyle1=2;"
   //"Timeframe4=-3;"
   //"QPoints=0;"
   //"InfoLevel=8;"
   + ParameterValues; // Append extra values set in input parameter
   
   int ret = ExtControlAdvanced1.Init(ExtCustomArgs1, UpdateRetouches, LogLevel, ValidateIncremental, ForceFullUpdate);
   
   if (UseExampleRSI && ret == INIT_SUCCEEDED)
   {
   #ifdef __MQL4__
      ExtRsiPath = "Examples\\RSI";
      ret = INIT_SUCCEEDED;
   #else
      ExtRsiPath = "Examples\\RSI.ex5";
      ExtRsiHandle = iCustom(_Symbol, _Period, ExtRsiPath, 14);
         
      ret = ExtRsiHandle > 0 ? INIT_SUCCEEDED : INIT_FAILED;
   #endif
   }
   
   if (LogLevel >= LogInfo) { Print("OnInit - ret=", ret); }
   return ret;
}


void OnDeinit(const int reason)
{
   DeleteCheck(ExtControlAdvanced1);
   DeleteCheck(ExtCustomArgs1);
   ChartRedraw();
}


void OnTick()
{
   datetime        time0 = iTime(NULL, 0, 0);                        // ###################################################################################
   SdEaUpdateEnum  sdChanged1 = ExtControlAdvanced1.Populate(time0); // ## For optimal performance - Edit Populate method and comment out unused variables
                                                                     // ###################################################################################
   if (sdChanged1 > SD_EA_UPDATE_NONE || ExtControlAdvanced1.NewBar)
   {
      if (sdChanged1 > SD_EA_UPDATE_NONE)
      {
         if (sdChanged1 == SD_EA_UPDATE_FULL)
         {
            // Add user code for full populate
            
         }
         // Add user code for incremental populate
         
         double   qHi1 = ExtControlAdvanced1.QHi1;
         double   qLo1 = ExtControlAdvanced1.QLow1;
         datetime currTime = TimeCurrent();

         if (ArraySize(ExtControlAdvanced1.SupLevels1) > 0)
         {
            SupDemLevel* sup1 = ExtControlAdvanced1.SupLevels1[0];
            double       sup1Hi = sup1.PriceHigh;
            double       sup1Low = sup1.PriceHigh;
            datetime     sup1Start = sup1.Timestamp;
            datetime     sup1End = currTime;
         }
      }
      if (ExtControlAdvanced1.NewBar)
      {
         // Add user code for new bar (current timeframe)
         
      }
   }
   // Add user code for each tick
   
   // Example code showing integration with RSI
   //
   if (UseExampleRSI)
   {
      int rsiHi = 55;
      int rsiLo = 45;
      double chartBid = iClose(_Symbol, _Period, 0);
      double currRSI = iCustomRSI(0, 0); // Get RSI using iCustom mechanism
      
      if (ExtRsiDirection != 1 && currRSI > rsiHi && ArraySize(ExtControlAdvanced1.SupLevels1) > 0)
      {
         SupDemLevel* sup1 = ExtControlAdvanced1.SupLevels1[0];
         
         if (chartBid > sup1.PriceLow - 50 * _Point)
         {
            Print("RSI (", DoubleToString(currRSI, _Digits), ") above ", rsiHi, " and bid price near first supply");
            ExtRsiDirection = 1;
         }
      }
      if (ExtControlAdvanced1.NewBar && ExtControlAdvanced1.NewSup1)
      {
         SupDemLevel* sup1 = ExtControlAdvanced1.SupLevels1[0];
         
         Print("New Supply formed ", TimeToString(sup1.Timestamp));
      }
      if (ExtRsiDirection != -1 && currRSI < rsiLo && ArraySize(ExtControlAdvanced1.DemLevels1) > 0)
      {
         SupDemLevel* dem1 = ExtControlAdvanced1.DemLevels1[0];
         
         if (chartBid < dem1.PriceHigh + 50 * _Point)
         {
            Print("RSI (", DoubleToString(currRSI, _Digits), ") below ", rsiLo, " and bid price near first demand");
            ExtRsiDirection = -1;
         }
      }
      if (ExtControlAdvanced1.NewBar && ExtControlAdvanced1.NewDem1)
      {
         SupDemLevel* dem1 = ExtControlAdvanced1.DemLevels1[0];
         
         Print("New Demand formed ", TimeToString(dem1.Timestamp));
      }
   }
}



double
iCustomRSI(int index, int shift)
{
#ifdef __MQL4__
   return
      iCustom(_Symbol, _Period, ExtRsiPath, 14, index, shift);
#else
   double arr[1];
   int    retIdx = CopyBuffer(ExtRsiHandle, index, shift, 1, arr);
   return retIdx > 0 ? arr[0] : EMPTY_VALUE;
#endif
}

