
#property description "Blahtech Supply Demand Expert Advisor - Basic"
#property version     "3.2"
#property strict

#include "BlahtechSupDemControlBasic.mqh"

#ifdef __MQL4__
input string             IndicatorPath = "Market\\Blahtech Supply Demand";          // Indicator Path
#else
input string             IndicatorPath = "Market\\Blahtech Supply Demand MT5.ex5";  // Indicator Path
#endif
input SdEaModeEnum       ExpertMode = SD_EA_EXPERT_NO_DRAW;                         // Expert Mode
input int                MinBarsLevel = 3;                                          // Min Bars For Level
input int                MaxBars = 20000;                                           // Max Bars
input int                ForceRecalcBars = 10000;                                   // Force Recalc Bars
input string             ParameterValues = "";                                      // Parameter Key Value Pairs
//input string             ParameterValues = "ZoneStyle1=2;InfoLevel=1 S;";         // Parameter Key Value Pairs
input MAPPED_TIMEFRAMES  Timeframe1 = MAP_PERIOD_CURRENT;                           // Timeframe 1
input MAPPED_TIMEFRAMES  Timeframe2 = MAP_PERIOD_MAPPED;                            // Timeframe 2
input MAPPED_TIMEFRAMES  Timeframe3 = MAP_PERIOD_MAPPED;                            // Timeframe 3
input SdEaRetouchMode    UpdateRetouches = SD_EA_RETOUCH_FIRST;                     // Update Retouches Mode
input SdEaValidateEnum   ValidateIncremental = SD_EA_VAL_NONE;                      // Validate Incremental
input bool               UseExampleRSI = false;                                     // Use Example RSI Integration Code
input LogLevelEnum       LogLevel = LogError;                                       // Log Level   


SupDemCustomArgs*    ExtCustomArgs1;
SupDemControlBasic*  ExtControlBasic1;
string               ExtRsiPath;
int                  ExtRsiHandle;
int                  ExtRsiDirection;


int OnInit()
{
   if (LogLevel >= LogInfo) { Print("OnInit - Start"); }
   
   ExtCustomArgs1 = new SupDemCustomArgs();
   ExtControlBasic1 = new SupDemControlBasic();
   
   ExtCustomArgs1.IndicatorPath = IndicatorPath;
   ExtCustomArgs1.ExpertMode = ExpertMode;
   ExtCustomArgs1.MinBarsLevel = MinBarsLevel;
   ExtCustomArgs1.MaxBars = MaxBars;
   ExtCustomArgs1.ForceRecalcBars = ForceRecalcBars;
   ExtCustomArgs1.Timeframe1 = Timeframe1;
   ExtCustomArgs1.Timeframe2 = Timeframe2;
   ExtCustomArgs1.Timeframe3 = Timeframe3;
   
   // ###################################################
   // ## Uncomment and/or modify to add parameter values
   // ###################################################
   ExtCustomArgs1.ParameterValues = "values:"
   "Hud=0;"
   //"SummaryMode=0;"
   //"TimeframeVisible2=1;"
   //"ZoneStyle1=2;"
   //"Timeframe4=-3;"
   //"QPoints=0;"
   //"InfoLevel=8;"
   + ParameterValues; // Append extra values set in input parameters
   
   int ret = ExtControlBasic1.Init(ExtCustomArgs1, UpdateRetouches, LogLevel, ValidateIncremental);
   
   if (UseExampleRSI && ret == INIT_SUCCEEDED)
   {
   #ifdef __MQL4__
      ExtRsiPath = "Examples\\RSI";
      ret = INIT_SUCCEEDED;
   #else
      ExtRsiPath = "Examples\\RSI.ex5";
      ExtRsiHandle = iCustom(_Symbol, _Period, ExtRsiPath, 14);
         
      ret = ExtRsiHandle > 0 ? INIT_SUCCEEDED : INIT_FAILED;
   #endif
   }

   if (LogLevel >= LogInfo) { Print("OnInit - ret=", ret); }
   return ret;
}


void OnDeinit(const int reason)
{
   DeleteCheck(ExtControlBasic1);
   DeleteCheck(ExtCustomArgs1);
   ChartRedraw();
}


void OnTick()
{
   datetime        time0 = iTime(NULL, 0, 0);                     // ###################################################################################
   SdEaUpdateEnum  sdChanged1 = ExtControlBasic1.Populate(time0); // ## For optimal performance - Edit Populate method and comment out unused variables
                                                                  // ###################################################################################
   if (sdChanged1 > SD_EA_UPDATE_NONE || ExtControlBasic1.NewBar)
   {
      if (sdChanged1 > SD_EA_UPDATE_NONE)
      {
         if (sdChanged1 == SD_EA_UPDATE_FULL)
         {
            // Add user code for full populate
            
         }
         // Add user code for incremental populate
         
         //double firstSupLevelHigh = ExtControlBasic1.SupPriceHigh;
         //double firstSupLevelLow = ExtControlBasic1.SupPriceLow;
         double qHi1 = ExtControlBasic1.QHi1;
         double qLo1 = ExtControlBasic1.QLow1;
      }
      
      if (ExtControlBasic1.NewBar)
      {
         // Add user code for new bar (current timeframe)
         
      }
   }
   // Add user code for each tick
   
   
   // Example code showing integration with RSI
   //
   if (UseExampleRSI)
   {
      int rsiHi = 55;
      int rsiLo = 45;
      double chartBid = iClose(_Symbol, _Period, 0);
      double currRSI = iCustomRSI(0, 0); // Get RSI using iCustom mechanism
      
      if (ExtRsiDirection != 1 && currRSI > rsiHi && ExtControlBasic1.HasSup)
      {
         if (chartBid > ExtControlBasic1.SupPriceLow - 50 * _Point)
         {
            Print("RSI (", DoubleToString(currRSI, _Digits), ") above ", rsiHi, " and bid price near first supply");
            ExtRsiDirection = 1;
         }
      }
      if (ExtControlBasic1.NewBar && ExtControlBasic1.NewSup1)
      {
         Print("New Supply formed ", TimeToString(ExtControlBasic1.SupTimestamp));
      }
      if (ExtRsiDirection != -1 && currRSI < rsiLo && ExtControlBasic1.HasDem)
      {
         if (chartBid < ExtControlBasic1.DemPriceHigh + 50 * _Point)
         {
            Print("RSI (", DoubleToString(currRSI, _Digits), ") below ", rsiLo, " and bid price near first demand");
            ExtRsiDirection = -1;
         }
      }
      if (ExtControlBasic1.NewBar && ExtControlBasic1.NewDem1)
      {
         Print("New Demand formed ", TimeToString(ExtControlBasic1.DemTimestamp));
      }
   }
}


double
iCustomRSI(int index, int shift)
{
#ifdef __MQL4__
   return iCustom(_Symbol, _Period, ExtRsiPath, 14, index, shift);
#else
   double arr[1];
   int    retIdx = CopyBuffer(ExtRsiHandle, index, shift, 1, arr);
   return retIdx > 0 ? arr[0] : EMPTY_VALUE;
#endif
}
