
#property description "VWAP Expert Advisor"
#property version     "1.1"
#property strict

enum VwEaModeEnum
{
   VW_EA_EXPERT = 1,          // EA - Draw       (Slower)
   VW_EA_EXPERT_NO_DRAW = 2,  // EA - No Drawing (Faster)
};

enum LogLevelEnum
{
   LogNone = 0,      // None
   LogError = 1,     // Error
   LogInfo = 2,      // Info
   LogDebug = 3,     // Debug
   LogAll = 4,       // All
};

enum VwBufferIndexEnum
{
   VW_VWAP = 0,
   VW_SETTLEMENT = 1,
   VW_UPPER_3 = 2,
   VW_UPPER_2 = 3,
   VW_UPPER_1 = 4,
   VW_LOWER_1 = 5,
   VW_LOWER_2 = 6,
   VW_LOWER_3 = 7,
};

enum VwEaUpdateEnum
{
   VW_EA_UPDATE_NONE = 0,
   VW_EA_UPDATE_CHANGED = 1,
};


#ifdef __MQL4__
input string         IndicatorPath = "Market\\Blahtech VWAP";             // Indicator Path
#else
input string         IndicatorPath = "Market\\Blahtech VWAP MT5.ex5";     // Indicator Path
#endif
input int            MaxBars = 20000;                                     // Max Bars
input bool           StdDevVisible = true;                                // Std Dev Visible
input bool           SettlementVisible = true;                            // Settlement Visible
input string         ParameterValues = "";                                // Parameter Key Value Pairs
input LogLevelEnum   LogLevel = LogError;                                 // Log Level

int    ExtIndicatorHandle;
string ExtParameterValues;


double iCustomVw(VwBufferIndexEnum index, int shift = 0)
{
#ifdef __MQL4__
   return iCustom(NULL, PERIOD_CURRENT, IndicatorPath, VW_EA_EXPERT, MaxBars, ExtParameterValues, index, shift);
#else
   double arr[1];
   return CopyBuffer(ExtIndicatorHandle, index, shift, 1, arr) > 0 ? arr[0] : EMPTY_VALUE;
#endif
}


int OnInit()
{
   // Example code for passing several parameters as key value pairs
   //
   ExtParameterValues = "values:"
   
   //
   // ### Comment out lines below to change parameters  ###
   // ### Or add new key value pairs as required        ###
   //
   //"VwInterval=0;"
   //"VwSessionStartTime=08:00;"
   //"VwSessionEndTime=22:00;"
   //"StdDevMethod=0;"
   //"SettlementVisible=0;"
   //"GMTOffset=0;"
   
   + ParameterValues // append key value pair parameters from input value
   + (StdDevVisible ? "StdDevVisible=1;" : "StdDevVisible=0;")
   + (SettlementVisible ? "SettlementVisible=1;" : "SettlementVisible=0;");
   

   Print("VW_EA::OnInit - ExtParameterValues=", ExtParameterValues);
   
#ifdef __MQL4__
   int  ret = INIT_SUCCEEDED;
#else
   ExtIndicatorHandle = iCustom(NULL, PERIOD_CURRENT, IndicatorPath, VW_EA_EXPERT, MaxBars, ExtParameterValues);
   int  ret = ExtIndicatorHandle > 0 ? INIT_SUCCEEDED : INIT_FAILED;
#endif
   if (LogLevel >= LogInfo) { Print("OnInit - IndicatorPath=", IndicatorPath); }
   return ret;
}


void OnDeinit(const int reason)
{
   #ifndef __MQL4__
   if (ExtIndicatorHandle > 0)
   {
      IndicatorRelease(ExtIndicatorHandle);
   }
   #endif
}


void OnTick()
{
   static int      stTickCount;
   static datetime stPrevTime0;
   static double   stPrevVwap;
   datetime        time0 = iTime(NULL, 0, 0);
   double          vwap = iCustomVw(VW_VWAP);

   stTickCount++;
   
   if (vwap != stPrevVwap || time0 != stPrevTime0)
   {
      bool  newBar = time0 != stPrevTime0;
      
      if (LogLevel >= LogDebug) { Print("OnTick TickCount=", stTickCount); }
      
      // For optimal performance - Comment out variables that are not required
      //
      if (SettlementVisible)
      {
         double settlememt = iCustomVw(VW_SETTLEMENT);
      }
      if (StdDevVisible)
      {
         double upper3 = iCustomVw(VW_UPPER_3);
         double upper2 = iCustomVw(VW_UPPER_2);
         double upper1 = iCustomVw(VW_UPPER_1);
         double lower1 = iCustomVw(VW_LOWER_1);
         double lower2 = iCustomVw(VW_LOWER_2);
         double lower3 = iCustomVw(VW_LOWER_3);
      }
      if (LogLevel >= LogAll) { Print("OnTick vwap=", DoubleToString(vwap, _Digits)); }
      
      if (newBar)
      {
         // Add user code for new bars
         
         stPrevTime0 = time0;  // Update prevTime0
      }
      // Add user code for each tick when VW Changed
      
      stPrevVwap = vwap; // Update previous value
   }
   // Add user code for each tick
}
